@@ -8,17 +8,17 @@
 #![doc(
     html_logo_url = "https://raw.githubusercontent.com/olson-sean-k/wax/master/doc/wax.svg?sanitize=true"
 )]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 
 mod capture;
+mod diagnostics;
 mod encode;
-mod fragment;
 mod rule;
-mod span;
 mod token;
 
 use bstr::ByteVec;
 use itertools::{Itertools as _, Position};
-#[cfg(feature = "diagnostics")]
+#[cfg(feature = "diagnostics-report")]
 use miette::Diagnostic;
 use regex::Regex;
 use std::borrow::{Borrow, Cow};
@@ -35,9 +35,19 @@ use walkdir::{self, DirEntry, WalkDir};
 
 pub use walkdir::Error as WalkError;
 
-use crate::token::Token;
+#[cfg(feature = "diagnostics-inspect")]
+use crate::diagnostics::inspect;
+#[cfg(feature = "diagnostics-report")]
+use crate::diagnostics::report::{self, BoxedDiagnostic};
+use crate::token::{Token, Tokenized};
 
-pub use crate::capture::Captures;
+pub use crate::capture::MatchedText;
+#[cfg(feature = "diagnostics-inspect")]
+pub use crate::diagnostics::inspect::CapturingToken;
+#[cfg(feature = "diagnostics-report")]
+pub use crate::diagnostics::report::{DiagnosticGlob, DiagnosticResult, DiagnosticResultExt};
+#[cfg(feature = "diagnostics-inspect")]
+pub use crate::diagnostics::Span;
 pub use crate::rule::RuleError;
 pub use crate::token::ParseError;
 
@@ -257,18 +267,24 @@ impl<T> Terminals<T> {
     }
 }
 
+// TODO: Is is not possible to use the `#[doc(cfg())]` attribute on the
+//       `Diagnostic` implementation generated by procedural macros.
+//       `Diagnostic` could be implemented without macros, but `GlobError` is
+//       particularly well suited to them. Use a manual implementation with
+//       `#[doc(cfg())]` if that changes to mark the implementation as feature
+//       dependent in documentation.
 #[derive(Debug, Error)]
 #[non_exhaustive]
-#[cfg_attr(feature = "diagnostics", derive(Diagnostic))]
+#[cfg_attr(feature = "diagnostics-report", derive(Diagnostic))]
 pub enum GlobError<'t> {
     #[error(transparent)]
-    #[cfg_attr(feature = "diagnostics", diagnostic(transparent))]
+    #[cfg_attr(feature = "diagnostics-report", diagnostic(transparent))]
     Parse(ParseError<'t>),
     #[error(transparent)]
-    #[cfg_attr(feature = "diagnostics", diagnostic(transparent))]
+    #[cfg_attr(feature = "diagnostics-report", diagnostic(transparent))]
     Rule(RuleError<'t>),
     #[error("failed to walk directory tree: {0}")]
-    #[cfg_attr(feature = "diagnostics", diagnostic(code = "glob::walk"))]
+    #[cfg_attr(feature = "diagnostics-report", diagnostic(code = "wax::glob::walk"))]
     Walk(WalkError),
 }
 
@@ -301,39 +317,39 @@ impl<'t> From<RuleError<'t>> for GlobError<'t> {
 }
 
 #[derive(Clone)]
-pub struct EncodedPath<'b> {
+pub struct CandidatePath<'b> {
     text: Cow<'b, str>,
 }
 
-impl<'b> EncodedPath<'b> {
-    pub fn into_owned(self) -> EncodedPath<'static> {
-        EncodedPath {
+impl<'b> CandidatePath<'b> {
+    pub fn into_owned(self) -> CandidatePath<'static> {
+        CandidatePath {
             text: self.text.into_owned().into(),
         }
     }
 }
 
-impl<'b> AsRef<str> for EncodedPath<'b> {
+impl<'b> AsRef<str> for CandidatePath<'b> {
     fn as_ref(&self) -> &str {
         self.text.as_ref()
     }
 }
 
-impl<'b> Debug for EncodedPath<'b> {
+impl<'b> Debug for CandidatePath<'b> {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         write!(f, "{:?}", self.text)
     }
 }
 
-impl<'b> Display for EncodedPath<'b> {
+impl<'b> Display for CandidatePath<'b> {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         write!(f, "{}", self.text)
     }
 }
 
-impl<'b> From<&'b OsStr> for EncodedPath<'b> {
+impl<'b> From<&'b OsStr> for CandidatePath<'b> {
     fn from(text: &'b OsStr) -> Self {
-        EncodedPath {
+        CandidatePath {
             text: match Vec::from_os_str_lossy(text) {
                 Cow::Borrowed(bytes) => str::from_utf8(bytes).expect_encoding().into(),
                 Cow::Owned(bytes) => String::from_utf8(bytes).expect_encoding().into(),
@@ -342,21 +358,21 @@ impl<'b> From<&'b OsStr> for EncodedPath<'b> {
     }
 }
 
-impl<'b> From<&'b Path> for EncodedPath<'b> {
+impl<'b> From<&'b Path> for CandidatePath<'b> {
     fn from(path: &'b Path) -> Self {
-        EncodedPath::from(path.as_os_str())
+        CandidatePath::from(path.as_os_str())
     }
 }
 
-impl<'b> From<&'b str> for EncodedPath<'b> {
+impl<'b> From<&'b str> for CandidatePath<'b> {
     fn from(text: &'b str) -> Self {
-        EncodedPath { text: text.into() }
+        CandidatePath { text: text.into() }
     }
 }
 
 #[derive(Clone, Debug)]
 pub struct Glob<'t> {
-    tokens: Vec<Token<'t>>,
+    tokenized: Tokenized<'t>,
     regex: Regex,
 }
 
@@ -369,20 +385,20 @@ impl<'t> Glob<'t> {
     }
 
     pub fn new(expression: &'t str) -> Result<Self, GlobError<'t>> {
-        let tokens = token::parse(expression)?;
-        let regex = Glob::compile(tokens.iter());
-        Ok(Glob { tokens, regex })
+        let tokenized = parse_and_check(expression)?;
+        let regex = Glob::compile(tokenized.tokens());
+        Ok(Glob { tokenized, regex })
     }
 
     /// Partitions a glob expression into an invariant `PathBuf` prefix and
     /// variant `Glob` postfix.
     ///
     /// The invariant prefix contains no glob patterns nor other variant
-    /// components and therefore can be interpretted as a native path. The
-    /// `Glob` postfix is variant and contains the remaining components (in
-    /// particular, patterns) that follow the prefix. For example, the glob
-    /// expression `.local/**/*.log` would produce the path `.local` and glob
-    /// `**/*.log`. It is possible for either partition to be empty.
+    /// components and therefore can be interpreted as a native path. The `Glob`
+    /// postfix is variant and contains the remaining components (in particular,
+    /// patterns) that follow the prefix. For example, the glob expression
+    /// `.local/**/*.log` would produce the path `.local` and glob `**/*.log`.
+    /// It is possible for either partition to be empty.
     ///
     /// Literal components may be considered variant if they contain characters
     /// with casing and the configured case sensitivity differs from the target
@@ -437,7 +453,7 @@ impl<'t> Glob<'t> {
     ///
     /// let path: &Path = /* ... */ // Candidate path.
     /// # Path::new("");
-    ///
+    ///     
     /// let directory = Path::new("."); // Working directory.
     /// let (prefix, glob) = Glob::partitioned("../../src/**").unwrap();
     /// let prefix = dunce::canonicalize(directory.join(&prefix)).unwrap();
@@ -455,60 +471,26 @@ impl<'t> Glob<'t> {
     /// prefix can be used to interact with native paths as needed for a given
     /// application.
     pub fn partitioned(expression: &'t str) -> Result<(PathBuf, Self), GlobError<'t>> {
-        // Get the invariant prefix for the token sequence.
-        let mut tokens = token::parse(expression)?;
-        let prefix = token::invariant_prefix_path(tokens.iter()).unwrap_or_else(PathBuf::new);
-
-        // Drain invariant tokens from the beginning of the token sequence.
-        // Unroot any tokens at the beginning of the sequence (tree wildcards).
-        tokens.drain(0..token::invariant_prefix_upper_bound(&tokens));
-        tokens.first_mut().map(Token::unroot);
-
-        let regex = Glob::compile(tokens.iter());
-        Ok((prefix, Glob { tokens, regex }))
+        let mut tokenized = parse_and_check(expression)?;
+        let prefix = tokenized.partition();
+        let regex = Glob::compile(tokenized.tokens());
+        Ok((prefix, Glob { tokenized, regex }))
     }
 
     pub fn into_owned(self) -> Glob<'static> {
-        let Glob { tokens, regex } = self;
-        let tokens = tokens.into_iter().map(|token| token.into_owned()).collect();
-        Glob { tokens, regex }
+        let Glob { tokenized, regex } = self;
+        Glob {
+            tokenized: tokenized.into_owned(),
+            regex,
+        }
     }
 
-    pub fn is_invariant(&self) -> bool {
-        // TODO: This may be expensive.
-        self.tokens
-            .iter()
-            .all(|token| token.to_invariant_string().is_some())
-    }
-
-    pub fn has_root(&self) -> bool {
-        self.tokens
-            .first()
-            .map(|token| token.is_rooted())
-            .unwrap_or(false)
-    }
-
-    #[cfg(any(unix, windows))]
-    pub fn has_semantic_literals(&self) -> bool {
-        token::components(self.tokens.iter()).any(|component| {
-            component
-                .literal()
-                .map(|literal| matches!(literal.text().as_ref(), "." | ".."))
-                .unwrap_or(false)
-        })
-    }
-
-    #[cfg(not(any(unix, windows)))]
-    pub fn has_semantic_literals(&self) -> bool {
-        false
-    }
-
-    pub fn is_match<'p>(&self, path: impl Into<EncodedPath<'p>>) -> bool {
+    pub fn is_match<'p>(&self, path: impl Into<CandidatePath<'p>>) -> bool {
         let path = path.into();
         self.regex.is_match(path.as_ref())
     }
 
-    pub fn captures<'p>(&self, path: &'p EncodedPath<'_>) -> Option<Captures<'p>> {
+    pub fn matched<'p>(&self, path: &'p CandidatePath<'_>) -> Option<MatchedText<'p>> {
         self.regex.captures(path.as_ref()).map(From::from)
     }
 
@@ -518,7 +500,7 @@ impl<'t> Glob<'t> {
         // invariant prefix from the glob pattern. `Walk` patterns are only
         // applied to path components following the `prefix` (distinct from the
         // glob pattern prefix) in `root`.
-        let (root, prefix, depth) = token::invariant_prefix_path(self.tokens.iter())
+        let (root, prefix, depth) = token::invariant_prefix_path(self.tokenized.tokens())
             .map(|prefix| {
                 let root = directory.join(&prefix).into();
                 if prefix.is_absolute() {
@@ -542,7 +524,7 @@ impl<'t> Glob<'t> {
                 let root = Cow::from(directory);
                 (root.clone(), root, depth)
             });
-        let regexes = Walk::compile(self.tokens.iter());
+        let regexes = Walk::compile(self.tokenized.tokens());
         Walk {
             regex: Cow::Borrowed(&self.regex),
             regexes,
@@ -553,6 +535,57 @@ impl<'t> Glob<'t> {
                 .into_iter(),
         }
     }
+
+    #[cfg(feature = "diagnostics-report")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "diagnostics-report")))]
+    pub fn diagnostics(&self) -> impl Iterator<Item = Box<dyn Diagnostic + '_>> {
+        report::diagnostics(&self.tokenized)
+    }
+
+    #[cfg(feature = "diagnostics-inspect")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "diagnostics-inspect")))]
+    pub fn captures(&self) -> impl '_ + Clone + Iterator<Item = CapturingToken> {
+        inspect::captures(self.tokenized.tokens())
+    }
+
+    pub fn is_invariant(&self) -> bool {
+        // TODO: This may be expensive.
+        self.tokenized
+            .tokens()
+            .iter()
+            .all(|token| token.to_invariant_string().is_some())
+    }
+
+    pub fn has_root(&self) -> bool {
+        self.tokenized
+            .tokens()
+            .first()
+            .map(|token| token.is_rooted())
+            .unwrap_or(false)
+    }
+
+    pub fn has_semantic_literals(&self) -> bool {
+        token::literals(self.tokenized.tokens()).any(|(_, literal)| literal.is_semantic_literal())
+    }
+}
+
+#[cfg_attr(docsrs, doc(cfg(feature = "diagnostics-report")))]
+#[cfg(feature = "diagnostics-report")]
+impl<'t> DiagnosticGlob<'t> for Glob<'t> {
+    fn new(expression: &'t str) -> DiagnosticResult<'t, Self> {
+        parse_and_diagnose(expression).map(|(tokenized, diagnostics)| {
+            let regex = Glob::compile(tokenized.tokens());
+            (Glob { tokenized, regex }, diagnostics)
+        })
+    }
+
+    fn partitioned(expression: &'t str) -> DiagnosticResult<'t, (PathBuf, Self)> {
+        parse_and_diagnose(expression).map(|(mut tokenized, diagnostics)| {
+            let prefix = tokenized.partition();
+            let regex = Glob::compile(tokenized.tokens());
+            ((prefix, Glob { tokenized, regex }), diagnostics)
+        })
+    }
 }
 
 impl<'t> TryFrom<&'t str> for Glob<'t> {
@@ -609,7 +642,7 @@ macro_rules! walk {
                 .components()
                 .skip(depth)
                 .filter_map(|component| match component {
-                    Component::Normal(component) => Some(EncodedPath::from(component)),
+                    Component::Normal(component) => Some(CandidatePath::from(component)),
                     _ => None,
                 })
                 .zip_longest($state.regexes.iter().skip(depth))
@@ -628,13 +661,13 @@ macro_rules! walk {
                     }
                     (Last(_) | Only(_), Both(component, regex)) => {
                         if regex.is_match(component.as_ref()) {
-                            let path = EncodedPath::from(path);
-                            if let Some(captures) =
-                                $state.regex.captures(path.as_ref()).map(Captures::from)
+                            let path = CandidatePath::from(path);
+                            if let Some(matched) =
+                                $state.regex.captures(path.as_ref()).map(MatchedText::from)
                             {
                                 let $entry = Ok(WalkEntry {
                                     entry: Cow::Borrowed(&entry),
-                                    captures,
+                                    matched,
                                 });
                                 $f
                             }
@@ -648,13 +681,13 @@ macro_rules! walk {
                         }
                     }
                     (_, Left(_component)) => {
-                        let path = EncodedPath::from(path);
-                        if let Some(captures) =
-                            $state.regex.captures(path.as_ref()).map(Captures::from)
+                        let path = CandidatePath::from(path);
+                        if let Some(matched) =
+                            $state.regex.captures(path.as_ref()).map(MatchedText::from)
                         {
                             let $entry = Ok(WalkEntry {
                                 entry: Cow::Borrowed(&entry),
-                                captures,
+                                matched,
                             });
                             $f
                             continue 'walk; // May be unreachable.
@@ -673,15 +706,15 @@ macro_rules! walk {
 #[derive(Debug)]
 pub struct WalkEntry<'e> {
     entry: Cow<'e, DirEntry>,
-    captures: Captures<'e>,
+    matched: MatchedText<'e>,
 }
 
 impl<'e> WalkEntry<'e> {
     pub fn into_owned(self) -> WalkEntry<'static> {
-        let WalkEntry { entry, captures } = self;
+        let WalkEntry { entry, matched } = self;
         WalkEntry {
             entry: Cow::Owned(entry.into_owned()),
-            captures: captures.into_owned(),
+            matched: matched.into_owned(),
         }
     }
 
@@ -696,7 +729,7 @@ impl<'e> WalkEntry<'e> {
         self.entry.path()
     }
 
-    pub fn to_encoded_path(&self) -> EncodedPath<'_> {
+    pub fn to_candidate_path(&self) -> CandidatePath<'_> {
         self.path().into()
     }
 
@@ -712,8 +745,8 @@ impl<'e> WalkEntry<'e> {
         self.entry.depth()
     }
 
-    pub fn captures(&self) -> &Captures<'e> {
-        &self.captures
+    pub fn matched(&self) -> &MatchedText<'e> {
+        &self.matched
     }
 }
 
@@ -788,7 +821,10 @@ impl<'g> Iterator for Walk<'g> {
     }
 }
 
-pub fn is_match<'p>(expression: &str, path: impl Into<EncodedPath<'p>>) -> Result<bool, GlobError> {
+pub fn is_match<'p>(
+    expression: &str,
+    path: impl Into<CandidatePath<'p>>,
+) -> Result<bool, GlobError> {
     let glob = Glob::new(expression)?;
     Ok(glob.is_match(path))
 }
@@ -844,6 +880,36 @@ pub const fn is_contextual_meta_character(x: char) -> bool {
     matches!(x, '-')
 }
 
+fn parse_and_check(expression: &str) -> Result<Tokenized, GlobError> {
+    let tokenized = token::parse(expression)?;
+    rule::check(&tokenized)?;
+    Ok(tokenized)
+}
+
+#[cfg(feature = "diagnostics-report")]
+fn parse_and_diagnose(expression: &str) -> DiagnosticResult<Tokenized> {
+    let (tokenized, parse_error_diagnostic) = match token::parse(expression) {
+        Ok(tokenized) => (Some(tokenized), None),
+        Err(diagnostic) => (None, Some(Box::new(diagnostic) as BoxedDiagnostic)),
+    };
+    let rule_error_diagnostic = tokenized.as_ref().and_then(|tokenized| {
+        rule::check(tokenized)
+            .err()
+            .map(|diagnostic| Box::new(diagnostic) as BoxedDiagnostic)
+    });
+    let non_error_diagnostics = tokenized.as_ref().into_iter().flat_map(report::diagnostics);
+    let diagnostics = non_error_diagnostics
+        .chain(rule_error_diagnostic)
+        .chain(parse_error_diagnostic)
+        .collect();
+    if let Some(tokenized) = tokenized {
+        Ok((tokenized, diagnostics))
+    }
+    else {
+        Err(diagnostics.try_into().expect("parse failed with no error"))
+    }
+}
+
 // TODO: Construct paths from components in tests. In practice, using string
 //       literals works, but is technically specific to platforms that support
 //       `/` as a separator.
@@ -851,7 +917,7 @@ pub const fn is_contextual_meta_character(x: char) -> bool {
 mod tests {
     use std::path::Path;
 
-    use crate::{Adjacency, EncodedPath, Glob, IteratorExt as _};
+    use crate::{Adjacency, CandidatePath, Glob, IteratorExt as _};
 
     #[test]
     fn adjacent() {
@@ -948,6 +1014,10 @@ mod tests {
         Glob::new("**/a").unwrap();
         Glob::new("a/**").unwrap();
         Glob::new("**/a/**/b/**").unwrap();
+        Glob::new("{**/a,b/c}").unwrap();
+        Glob::new("{a/b,c/**}").unwrap();
+        Glob::new("<**/a>").unwrap();
+        Glob::new("<a/**>").unwrap();
     }
 
     #[test]
@@ -1041,6 +1111,7 @@ mod tests {
         assert!(Glob::new("***").is_err());
         assert!(Glob::new("****").is_err());
         assert!(Glob::new("**/**").is_err());
+        assert!(Glob::new("a{**/**,/b}").is_err());
         assert!(Glob::new("**/*/***").is_err());
         assert!(Glob::new("**$").is_err());
         assert!(Glob::new("**/$**").is_err());
@@ -1185,9 +1256,9 @@ mod tests {
         // text should always be available.
         assert_eq!(
             "a/b",
-            glob.captures(&EncodedPath::from(Path::new("a/b")))
+            glob.matched(&CandidatePath::from(Path::new("a/b")))
                 .unwrap()
-                .matched(),
+                .complete(),
         );
     }
 
@@ -1204,7 +1275,7 @@ mod tests {
 
         assert_eq!(
             "x/y/z/",
-            glob.captures(&EncodedPath::from(Path::new("a/x/y/z/b")))
+            glob.matched(&CandidatePath::from(Path::new("a/x/y/z/b")))
                 .unwrap()
                 .get(1)
                 .unwrap(),
@@ -1219,10 +1290,10 @@ mod tests {
         assert!(glob.is_match(Path::new("a/file.ext")));
         assert!(glob.is_match(Path::new("a/b/file.ext")));
 
-        let path = EncodedPath::from(Path::new("a/file.ext"));
-        let captures = glob.captures(&path).unwrap();
-        assert_eq!("a/", captures.get(1).unwrap());
-        assert_eq!("file", captures.get(2).unwrap());
+        let path = CandidatePath::from(Path::new("a/file.ext"));
+        let matched = glob.matched(&path).unwrap();
+        assert_eq!("a/", matched.get(1).unwrap());
+        assert_eq!("file", matched.get(2).unwrap());
     }
 
     #[test]
@@ -1232,11 +1303,11 @@ mod tests {
         assert!(glob.is_match(Path::new("prefix-file.ext")));
         assert!(glob.is_match(Path::new("a-b-c.ext")));
 
-        let path = EncodedPath::from(Path::new("a-b-c.ext"));
-        let captures = glob.captures(&path).unwrap();
-        assert_eq!("a", captures.get(1).unwrap());
-        assert_eq!("b-c", captures.get(2).unwrap());
-        assert_eq!("ext", captures.get(3).unwrap());
+        let path = CandidatePath::from(Path::new("a-b-c.ext"));
+        let matched = glob.matched(&path).unwrap();
+        assert_eq!("a", matched.get(1).unwrap());
+        assert_eq!("b-c", matched.get(2).unwrap());
+        assert_eq!("ext", matched.get(3).unwrap());
     }
 
     #[test]
@@ -1249,9 +1320,9 @@ mod tests {
 
         assert!(!glob.is_match(Path::new("a/b/file.ext")));
 
-        let path = EncodedPath::from(Path::new("a/i/file.ext"));
-        let captures = glob.captures(&path).unwrap();
-        assert_eq!("i", captures.get(1).unwrap());
+        let path = CandidatePath::from(Path::new("a/i/file.ext"));
+        let matched = glob.matched(&path).unwrap();
+        assert_eq!("i", matched.get(1).unwrap());
     }
 
     #[test]
@@ -1263,9 +1334,9 @@ mod tests {
 
         assert!(!glob.is_match(Path::new("a/銅/file.ext")));
 
-        let path = EncodedPath::from(Path::new("a/金/file.ext"));
-        let captures = glob.captures(&path).unwrap();
-        assert_eq!("金", captures.get(1).unwrap());
+        let path = CandidatePath::from(Path::new("a/金/file.ext"));
+        let matched = glob.matched(&path).unwrap();
+        assert_eq!("金", matched.get(1).unwrap());
     }
 
     #[test]
@@ -1278,9 +1349,9 @@ mod tests {
 
         assert!(!glob.is_match(Path::new("a/b/file.ext")));
 
-        let path = EncodedPath::from(Path::new("a/[/file.ext"));
-        let captures = glob.captures(&path).unwrap();
-        assert_eq!("[", captures.get(1).unwrap());
+        let path = CandidatePath::from(Path::new("a/[/file.ext"));
+        let matched = glob.matched(&path).unwrap();
+        assert_eq!("[", matched.get(1).unwrap());
     }
 
     #[cfg(any(unix, windows))]
@@ -1305,18 +1376,18 @@ mod tests {
         assert!(!glob.is_match(Path::new("a/y/file.ext")));
         assert!(!glob.is_match(Path::new("a/xyzub/file.ext")));
 
-        let path = EncodedPath::from(Path::new("a/xyzb/file.ext"));
-        let captures = glob.captures(&path).unwrap();
-        assert_eq!("xyz", captures.get(1).unwrap());
+        let path = CandidatePath::from(Path::new("a/xyzb/file.ext"));
+        let matched = glob.matched(&path).unwrap();
+        assert_eq!("xyz", matched.get(1).unwrap());
     }
 
     #[test]
     fn match_glob_with_nested_alternative_tokens() {
         let glob = Glob::new("a/{y$,{x?z,?z}}b/*").unwrap();
 
-        let path = EncodedPath::from(Path::new("a/xyzb/file.ext"));
-        let captures = glob.captures(&path).unwrap();
-        assert_eq!("xyz", captures.get(1).unwrap());
+        let path = CandidatePath::from(Path::new("a/xyzb/file.ext"));
+        let matched = glob.matched(&path).unwrap();
+        assert_eq!("xyz", matched.get(1).unwrap());
     }
 
     #[test]
@@ -1352,9 +1423,9 @@ mod tests {
         assert!(!glob.is_match(Path::new("a/0000000/file.ext")));
         assert!(!glob.is_match(Path::new("a/bbbbbb/file.ext")));
 
-        let path = EncodedPath::from(Path::new("a/999999/file.ext"));
-        let captures = glob.captures(&path).unwrap();
-        assert_eq!("999999", captures.get(1).unwrap());
+        let path = CandidatePath::from(Path::new("a/999999/file.ext"));
+        let matched = glob.matched(&path).unwrap();
+        assert_eq!("999999", matched.get(1).unwrap());
     }
 
     #[test]
@@ -1378,9 +1449,9 @@ mod tests {
         assert!(!glob.is_match(Path::new("log-abc.txt")));
         assert!(!glob.is_match(Path::new("log-123-456-789.txt")));
 
-        let path = EncodedPath::from(Path::new("log-987-654.txt"));
-        let captures = glob.captures(&path).unwrap();
-        assert_eq!("-987-654", captures.get(1).unwrap());
+        let path = CandidatePath::from(Path::new("log-987-654.txt"));
+        let matched = glob.matched(&path).unwrap();
+        assert_eq!("-987-654", matched.get(1).unwrap());
     }
 
     #[test]
@@ -1395,9 +1466,9 @@ mod tests {
         assert!(!glob.is_match(Path::new("file.ext")));
         assert!(!glob.is_match(Path::new("c/file.ext")));
 
-        let path = EncodedPath::from(Path::new("aa/file.ext"));
-        let captures = glob.captures(&path).unwrap();
-        assert_eq!("aa", captures.get(1).unwrap());
+        let path = CandidatePath::from(Path::new("aa/file.ext"));
+        let matched = glob.matched(&path).unwrap();
+        assert_eq!("aa", matched.get(1).unwrap());
     }
 
     #[test]
@@ -1410,9 +1481,9 @@ mod tests {
         assert!(!glob.is_match(Path::new("./var/cron.log")));
         assert!(!glob.is_match(Path::new("mnt/var/log/cron.log")));
 
-        let path = EncodedPath::from(Path::new("/var/log/network.log"));
-        let captures = glob.captures(&path).unwrap();
-        assert_eq!("/", captures.get(1).unwrap());
+        let path = CandidatePath::from(Path::new("/var/log/network.log"));
+        let matched = glob.matched(&path).unwrap();
+        assert_eq!("/", matched.get(1).unwrap());
     }
 
     #[test]
@@ -1581,6 +1652,9 @@ mod tests {
     fn query_glob_has_semantic_literals() {
         assert!(Glob::new("../src/**").unwrap().has_semantic_literals());
         assert!(Glob::new("*/a/../b.*").unwrap().has_semantic_literals());
+        assert!(Glob::new("{a,..}").unwrap().has_semantic_literals());
+        assert!(Glob::new("<a/..>").unwrap().has_semantic_literals());
+        assert!(Glob::new("<a/{b,..,c}/d>").unwrap().has_semantic_literals());
         assert!(Glob::new("./*.txt").unwrap().has_semantic_literals());
     }
 }
